#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.

# -*- coding: utf-8 -*-
"""
Script for carrying out analysis on binary mixtures.

@author: mark
"""

import logging
import pandas
import resultsanalysis.resultsoutput.plottinginput as pltinp

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

DEFAULT_MARKER_STYLES = [{'marker1':"o",
                  'mec1':"k",
                  'mfc1': None,
                  'marker2':"v",
                  "mec2":"k",
                  "mfc2": None,
                  },
                         {'marker1':"None",
                  'mec1':"b",
                  'mfc1': None,
                  'marker2':"None",
                  "mec2":"r",
                  "mfc2": None,
                  'linestyle': "solid",
                  'linecolor': "r",
                  },
                         {'marker1':"None",
                  'mec1':"b",
                  'mfc1': None,
                  'marker2':"None",
                  "mec2":"r",
                  "mfc2": None,
                  'linestyle': "dashed",
                  'linecolor': "r",
                  },
                         {'marker1':"None",
                  'mec1':"b",
                  'mfc1': None,
                  'marker2':"None",
                  "mec2":"r",
                  "mfc2": None,
                  'linestyle': "dotted",
                  'linecolor': "r",
                  'linewidth': 2.0,
                  },
                         {'marker1':"None",
                  'mec1':"b",
                  'mfc1': None,
                  'marker2':"None",
                  "mec2":"r",
                  "mfc2": None,
                  'linestyle': "dotted",
                  'linecolor': "b",
                  'linewidth': 2.0,
                  },
                         {'marker1':"None",
                  'mec1':"b",
                  'mfc1': None,
                  'marker2':"None",
                  "mec2":"r",
                  "mfc2": None,
                  'linestyle': "dashed",
                  'linecolor': "b",
                  },
                         {'marker1':"None",
                  'mec1':"b",
                  'mfc1': None,
                  'marker2':"None",
                  "mec2":"r",
                  "mfc2": None,
                  'linestyle': "solid",
                  'linecolor': "b",
                  },
                         ]

def plot_distances_by_domain(total_filename, domain_filename_list, outputfile_stem, axis_info, fileformat="eps", **kwargs):
    """

    Parameters
    ----------
    total_filename : TYPE
        DESCRIPTION.
    domain_filename_list : TYPE
        DESCRIPTION.
    outputfile_stem : TYPE
        DESCRIPTION.
    axis_info : TYPE
        DESCRIPTION.
    fileformat : TYPE, optional
        DESCRIPTION. The default is "eps".
    **kwargs : TYPE
        DESCRIPTION.

    Returns
    -------
    None.

    """
    input_data_list = []
    if total_filename is not None:
        input_data_list = create_input_data_list(total_filename, **DEFAULT_MARKER_STYLES[0])
    for domainid, domain_filename in enumerate(domain_filename_list):
        input_data_list.extend(create_input_data_list(domain_filename, **DEFAULT_MARKER_STYLES[domainid +1]))
    LOGGER.info("information read in.")
    LOGGER.debug("input data list: ")
    LOGGER.debug(input_data_list)
    pltinp.create_scatter_plot_multidata(input_data_list, outputfile_stem, axis_info, fileformat=fileformat)

def create_input_data_list(filename, **kwargs):
    """

    Parameters
    ----------
    filename : TYPE
        DESCRIPTION.
    **kwargs : TYPE
        DESCRIPTION.

    Returns
    -------
    list
        DESCRIPTION.

    """
    dist_matrix = read_matrix(filename)
    dist_to_comp1, dist_to_comp2 = get_dist_to_pure_comps(dist_matrix, kwargs.pop("component1", None), kwargs.pop("component2", None), label_type=kwargs.pop("label_type", None))
    comp1_dict = {'x_data':[float(comp)/100.0 for comp in dist_to_comp1.index.to_list()],
                  'y_data':dist_to_comp1.to_list(),
                  'marker':kwargs.pop("marker1","o"),
                  'mec':kwargs.pop("mec1","b"),
                  'mfc':kwargs.pop("mfc1", None),
                  'linestyle':kwargs.get('linestyle','None'),
                  'linecolor':kwargs.get('linecolor','None'),
                  'linewidth':kwargs.get('linewidth',2.0),
                  }
    comp2_dict = {'x_data':[float(comp)/100.0 for comp in dist_to_comp2.index.to_list()],
                  'y_data':dist_to_comp2.to_list(),
                  'marker': kwargs.pop("marker2","v"),
                  'mec':kwargs.pop("mec2","b"),
                  'mfc':kwargs.pop("mfc2", None),
                  'linestyle': kwargs.get('linestyle','None'),
                  'linecolor':kwargs.get('linecolor','None'),
                  'linewidth':kwargs.get('linewidth',2.0),
                  }
    return [comp1_dict, comp2_dict]

def get_dist_to_pure_comps(dataframe, component1, component2, label_type="percent"):
    """

    Parameters
    ----------
    dataframe : TYPE
        DESCRIPTION.
    component1 : TYPE
        DESCRIPTION.
    component2 : TYPE
        DESCRIPTION.
    label_type : TYPE, optional
        DESCRIPTION. The default is "percent".

    Returns
    -------
    dist_to_comp1 : TYPE
        DESCRIPTION.
    dist_to_comp2 : TYPE
        DESCRIPTION.

    """
    #assuming numbers are written as percentages:
    dist_to_comp1 = dataframe.loc[:,'0']
    dist_to_comp2 = dataframe.loc[:,'100']
    #TODO update to make this more general
    return dist_to_comp1, dist_to_comp2

def read_matrix(filename):
    """

    Parameters
    ----------
    filename : TYPE
        DESCRIPTION.

    Returns
    -------
    TYPE
        DESCRIPTION.

    """
    return pandas.read_csv(filename, sep="\t", index_col=0)