#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Created on Mon Jul 31 22:02:14 2017

@author: mark
"""

import logging
import matplotlib.pyplot as plt
import scipy.cluster.hierarchy as cluster
import resultsanalysis.resultsoutput.plottinginput as plottinginput

plottinginput.plt.tight_layout()

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.INFO)


def create_and_write_dendogram(linkage_matrix, labels, filename_stem, **kwargs):
    """This creates a dendogram plot and writes to file.
    """
    dendrogram_plot, dendrogram_plot_axis, y_ticks = create_dendogram(
        linkage_matrix, labels, kwargs.pop("figsize", (25, 20)), **kwargs
    )
    append_plot_labels(dendrogram_plot_axis, **kwargs)
    fileformat = kwargs.get("fileformat", "eps")
    output_filename = create_outputfilename(filename_stem, fileformat)
    plt.subplots_adjust(left=kwargs.get("left_margin",0.25), right=kwargs.get("right_margin",0.98),
                        top=kwargs.get("top_margin",0.95),bottom=kwargs.get("bottom_margin",0.1))
    LOGGER.info("Tick labels")
    LOGGER.info(y_ticks)
    plt.savefig(output_filename, format=fileformat)
    plt.close(dendrogram_plot)
    return y_ticks


def create_outputfilename(filename_stem, fileformat):
    """This creates a filename from the stem and file format.
    """
    return plottinginput.create_output_filename_from_stem(filename_stem, fileformat)


def create_dendogram(linkage_matrix, labels, figsize, **kwargs):
    """Creates a dendogram plot.
    """
    dendrogram_plot, dendrogram_plot_axis = create_plot_with_axis(figsize)
    threshold = 0.4 * max(linkage_matrix[:, 2])
    cluster.dendrogram(
        linkage_matrix,
        labels=labels,
        orientation="right",
        leaf_rotation=0,
        leaf_font_size=kwargs.pop("leaf_font_size", 7),
        color_threshold=threshold,
        ax=dendrogram_plot_axis,
    )
    append_plot_labels(dendrogram_plot_axis, **kwargs)
    dendrogram_plot_axis.tick_params(axis="x", labelsize=14)
    for label in dendrogram_plot_axis.yaxis.get_ticklabels():
        label.set_horizontalalignment("right")
    if kwargs.get("plot_vline", False):
        dendrogram_plot_axis.axvline(x=threshold, c="k")
    y_label_list = dendrogram_plot_axis.get_yticklabels()
    y_label_list = [y_label.get_text() for y_label in y_label_list]
    return dendrogram_plot, dendrogram_plot_axis, y_label_list


def append_plot_labels(plot_axis, **kwargs):
    """This adds labels to the dendrogram axis.
    """
    LOGGER.info("y label: %s", kwargs.get("ylabel", "Solvent Name"))
    if kwargs.get("ylabel", "Solvent Name") is not None:
        plot_axis.set_ylabel(
            kwargs.pop("ylabel", "Solvent Name"),
            fontsize=kwargs.get("label_fontsize", 20),
        )
    plot_axis.set_xlabel(
        kwargs.pop("xlabel", "Solvent Similarity Index"),
        fontsize=kwargs.get("label_fontsize", 20),
    )
    plot_axis.tick_params(axis='x', labelsize=kwargs.get("label_fontsize", 20))


def create_plot_with_axis(figsize):
    """Function creates plot and plot axis.
    """
    return plottinginput.create_plot_with_axis(figsize)
