#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for preparing the Similarity Matrices for clustering. This contains
methods to reads in a matrix from a file, and return the labels as well as a
triangular matrix ready for clustering.

@author: mark
"""

import logging
import scipy.spatial.distance as scidist
import solventmapcreator.io.similaritymatrixreader as similaritymatrixreader

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def parse_similarity_matrix_file_condensed_matrix(filename):
    """This reads a file into a dictionary, then appends the condensed matrix
    to the dictionary.
    """
    similarity_matrix_info = parse_similarity_matrix_file(filename)
    append_condensed_matrix(similarity_matrix_info)
    return similarity_matrix_info


def append_condensed_matrix(similarity_matrix_info):
    """This appends a condensed matrix to the input dict.
    """
    similarity_matrix_info["condensed_matrix"] = convert_to_condensed_matrix(
        similarity_matrix_info["value_matrix"]
    )


def convert_to_condensed_matrix(similarity_matrix):
    """This converts the matrix to a condensed matrix.
    """
    return scidist.squareform(similarity_matrix)


def parse_similarity_matrix_file(filename):
    """This reads a file in to a dictionary, containing the labels and the
    similarity matrix.
    """
    return similaritymatrixreader.parse_similarity_matrix_file(filename)
