#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for writing out the similarity matrix information to a csv file.

@author: mark
"""

import logging
import csv

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def write_similarity_matrix_file(filename, input_data):
    """This writes out the similarity matrix information to file.
    """
    with open(filename, "w") as outfile:
        csv_writer = csv.writer(outfile, delimiter="\t", quoting=csv.QUOTE_NONE,lineterminator="\n")
        csv_writer.writerow(["", *input_data["labels"]])
        similarity_matrix = input_data["value_matrix"]
        for i in range(len(input_data["labels"])):
            data_line = create_file_line(input_data["labels"][i], similarity_matrix[i])
            csv_writer.writerow(data_line)
    return 0


def create_file_line(solvent_name, similarity_matrix_row):
    """This creates the row, ready to write to file. This formats the similarity
    matrix values ofr writing.
    """
    similarity_matrix_row_values = ["{:.7f}".format(x) for x in similarity_matrix_row]
    return [solvent_name, *similarity_matrix_row_values]
