#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for the reading in of solvation energies for a given solvent.

@author: mark
"""

import logging
import solventmapcreator.io.solvationreading as solvationreader

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def parse_free_energy_from_file(filename):
    """This parses the file and returns a datapoints representation of it.
    """
    solvation_energy_dict = parse_free_energy_info(filename)
    return create_free_energy_datapoints_with_values(solvation_energy_dict)


def parse_binding_energy_from_file(filename):
    """This parses the file and returns a datapoints representation of it.
    """
    solvation_energy_dict = parse_binding_energy_info(filename)
    return create_binding_datapoints_with_values(solvation_energy_dict)


def create_free_energy_datapoints_with_values(solvation_energy_dict):
    """This creates free energy datapoints.
    """
    return solvationreader.create_free_energy_datapoints_with_values(
        solvation_energy_dict
    )


def create_binding_datapoints_with_values(solvation_energy_dict):
    """This creates binding datapoints
    """
    return solvationreader.create_binding_datapoints_with_values(solvation_energy_dict)


def parse_free_energy_info(filename):
    """This extracts the solvation information to a dictionary.
    """
    return solvationreader.parse_free_energy_info(filename)


def parse_binding_energy_info(filename):
    """This extracts the solvation information to a dictionary.
    """
    return solvationreader.parse_binding_energy_info(filename)
