#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for writing out the solvation values generated for a solvent map.

Format is:

    Y1 Y2 ...
X1 S(X1,Y1) S(X1,Y2) ...
X2 S(X2,Y1) S(X2,Y2) ...
.  .
.  .
.  .
XN S(XN,Y1) S(XN,Y2) ...

@author: mark

"""

import logging
import csv

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def write_matrix_to_file(filename, x_values, y_values, solvation_matrix):
    """This writes the matrix to file.
    """
    with open(filename, "w") as outfile:
        csv_writer = csv.writer(outfile, delimiter="\t", quoting=csv.QUOTE_NONE,lineterminator="\n")
        csv_writer.writerow(["", *y_values])
        for i in range(len(x_values)):
            csv_writer.writerow(write_matrix_line(x_values[i], solvation_matrix[i]))
    return 0


def write_matrix_line(x_value, solvation_matrix_row):
    """This creates the list of entries ready for writing.
    """
    return [
        "{:.3f}".format(x_value),
        *["{:.7f}".format(val) for val in solvation_matrix_row],
    ]
