#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for plotting the similarity Matrix information as a graph, for
graphical comparison.

@author: mark
"""

import logging
from matplotlib import cm as cm
import matplotlib.pyplot as plt
import resultsanalysis.resultsoutput.plottinginput as plottinginput

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


def create_similarity_matrix_plot(input_data, filename_stem, **kwargs):
    """This creates the similarity plot for the given data and outputs to file.
    """
    simil_plot, simil_plot_axis = plot_similarity_matrix(input_data, **kwargs)
    fileformat = kwargs.get("fileformat", "eps")
    output_filename = filename_stem + "." + fileformat
    plt.savefig(output_filename, format=fileformat)
    plt.close(simil_plot)


def plot_similarity_matrix(input_data, **kwargs):
    """This creates the similarity plot for the given input data.
    """
    cmap = cm.get_cmap("plasma")
    simil_plot, simil_plot_axis = create_plot_with_axis(
        kwargs.get("figsize", plottinginput.FIGURE_SIZES["similarity matrix"])
    )
    mat_ax = simil_plot_axis.matshow(
        input_data["value_matrix"], interpolation="nearest", cmap=cmap
    )
    simil_plot_axis.grid(True)
    simil_plot_axis.set_xticks(range(len(input_data["labels"])))
    simil_plot_axis.set_yticks(range(len(input_data["labels"])))
    simil_plot_axis.set_xticklabels(input_data["labels"], rotation=90)
    simil_plot_axis.set_yticklabels(input_data["labels"])
    simil_plot_axis.set_xlabel("Solvent")
    simil_plot_axis.set_ylabel("Solvent")
    simil_plot.colorbar(mat_ax)
    return simil_plot, simil_plot_axis


def create_plot_with_axis(figsize):
    """This creates the plot and axis.
    """
    return plottinginput.create_plot_with_axis(figsize)
