# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Created on Sun Aug 13 22:42:58 2017

@author: mark
"""

import logging
import numpy as np
import os
import unittest
import pathlib
import solventmapcreator.io.solvationmapwriter as solvmapwriter

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class SolvationMapWriterTestCase(unittest.TestCase):
    """Test case for the solvation
    """

    def setUp(self):
        """Set up for tests.
        """
        parent_directory = pathlib.Path(__file__).parents[1]
        self.simil_file = (
            (parent_directory / "resources/similmatrix.csv").absolute().as_posix()
        )
        self.x_list = np.array([float(x) for x in range(3)])
        self.y_list = np.array([float(x) for x in range(2)])
        self.similarity_matrix = np.array(
            [[-4.332289, -5.403718], [-5.403718, -5.757629], [-6.475146, -6.167992]]
        )

    def tearDown(self):
        """Clean up after tests
        """
        del self.x_list
        del self.y_list
        del self.similarity_matrix
        if os.path.isfile("actualsimilmatrix.csv"):
            os.remove("actualsimilmatrix.csv")

    def test_write_similarity_matrix_file(self):
        """Test to see if expected file is written out.
        """
        actual_file_name = "actualsimilmatrix.csv"
        simil_out_file = solvmapwriter.write_matrix_to_file(
            actual_file_name, self.x_list, self.y_list, self.similarity_matrix
        )
        self.assertEqual(0, simil_out_file)
        with open(self.simil_file, "r") as exp_file:
            exp_file_lines = exp_file.readlines()
            with open(actual_file_name, "r") as act_file:
                act_file_lines = act_file.readlines()
                self.assertListEqual(act_file_lines, exp_file_lines)

    def test_create_file_line(self):
        """Test to see if expected list is returned.
        """
        expected_list = ["0.000", "-4.3322890", "-5.4037180"]
        actual_list = solvmapwriter.write_matrix_line(
            self.x_list[0], self.similarity_matrix[0]
        )
        self.assertListEqual(expected_list, actual_list)
