# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for testing the fractionaloccupancycalculator

@author: mark
"""

import logging
import unittest
import pathlib
import solventmapcreator.solvationcalculation.fractionaloccupancycalculator as fractionaloccupancycalculator

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class FractionalOccupancyCalculatorTestCase(unittest.TestCase):
    """Test case for the fractionaloccupancycalculator script."""

    def setUp(self):
        """Set up for tests."""
        parent_directory = pathlib.Path(__file__).parents[1]
        self.frac_occ_file = (parent_directory / "resources/fractional_occupancy.xml").absolute().as_posix()
        self.frac_occ_inf_dict = fractionaloccupancycalculator.read_fractional_occupancy_information(self.frac_occ_file)
    def tearDown(self):
        """Clean up after tests."""

    def test_calculate_fractional_occupancy_for_phase(self):
        """Test to see if expected value is returned."""
        expected_value = 0.7380033333333333
        actual_value = fractionaloccupancycalculator.calculate_fractional_occupancy_for_phase(
            self.frac_occ_inf_dict, "water", 298.0
        )
        self.assertAlmostEqual(expected_value, actual_value)

    def test_read_fractional_occupancy_information(self):
        """Test to see if fractional occupancy is as expected."""
        expected_condensed_dict = {"water298.000":0.7380033333333333}
        self.assertDictEqual(expected_condensed_dict, self.frac_occ_inf_dict)
