# -*- coding: utf-8 -*-
#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
"""
Script for testing solvationcalculator script.

@author: mark
"""

import logging
import unittest
import numpy as np
import solventmapcreator.solvationcalculation.solvationcalculator as solvationcalculator

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)


class SolvationCalculatorTestCase(unittest.TestCase):
    """Test case for the solvationcalculator script
    """

    def setUp(self):
        """Set up for tests.
        """

    def tearDown(self):
        """tear down after tests.
        """

    def test_calcluate_association_energy_matrix(self):
        """Test to see if expected matrix is produced.
        """
        expected_array = np.array([[-9.021987]])
        actual_array = solvationcalculator.calcluate_association_energy_matrix(
            [1.0], [1.0], 298.0, 1.0, np.array([0.5, 1.0])
        )
        np.testing.assert_array_almost_equal(expected_array, actual_array)
        actual_array2 = solvationcalculator.calcluate_association_energy_matrix(
            np.array([1.0]), np.array([1.0]), 298.0, 1.0, np.array([0.5, 1.0])
        )
        np.testing.assert_array_almost_equal(expected_array, actual_array2)

    def test_calculate_association_energy(self):
        """Test to see if expected_value is returned.
        """
        expected_value = -9.0219865959507608
        actual_value = solvationcalculator.calculate_association_energy(
            1.0, 1.0, 298.0, 1.0, np.array([0.5, 1.0])
        )
        self.assertAlmostEqual(expected_value, actual_value)

    def test_calculate_binding_energy(self):
        """Test to see if expected binding energy is returned
        """
        expected_value = -6.0219865959507599
        actual_value = solvationcalculator.calculate_binding_energy(
            1.0, 1.0, 298.0, 1.0
        )
        self.assertAlmostEqual(expected_value, actual_value)

    def test_calculate_psi_vdw(self):
        """Test to see if expected value is returned.

        Returns
        -------
        None.

        """
        expected_value = 0.2874159883815648
        actual_value = solvationcalculator.calculate_psi_vdw(298, 0.9)
        self.assertAlmostEqual(expected_value, actual_value)

    def test_calculate_confinement_energy(self):
        """Test to see if expected value is calculated.
        """
        expected_value = 1.7174170217382039
        actual_value = solvationcalculator.calculate_confinement_energy(298.0, 1.0)
        self.assertAlmostEqual(expected_value, actual_value)

    def test_calculate_solvation_energy(self):
        """Test to see if expected solvation energy is returned.
        """
        expected_value = 1.0
        actual_value = solvationcalculator.calculate_solvation_energy(
            0.5, np.array([0.5, 1.0])
        )
        self.assertAlmostEqual(expected_value, actual_value)

    def test_calculate_association_constant(self):
        """Test to see if expected value is returned.
        """
        expected_value = 7.9930934618579919
        actual_value = solvationcalculator.calculate_association_constant(
            1.0, 1.0, 298.0
        )
        self.assertAlmostEqual(expected_value, actual_value)

    def test_calculate_assoc_const(self):
        """Test to see if expected association constant is returned.
        """
        expected_value = 3.200818570079262
        actual_value = solvationcalculator.calculate_assoc_const(1.0, 1.0, 298.0)
        self.assertAlmostEqual(expected_value, actual_value)
