#    solventmapcreator provides tools for calculation of FGIPs and solvent similarity
#    Copyright (C) 2019  Mark D. Driver
#
#    solventmapcreator is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <https://www.gnu.org/licenses/>.
# -*- coding: utf-8 -*-
"""
Script for running the tests.

@author: mark
"""

import logging
import unittest
import sys
from solventmapcreator.test.iotest.solvationenergyreadertest import (
    SolvationEnergyReaderTestCase,
)
from solventmapcreator.test.iotest.polynomialdatawritertest import (
    PolynomialDataWriterTestCase,
)
from solventmapcreator.test.iotest.polynomialdatareadertest import (
    PolynomialDataReaderTestCase,
)
from solventmapcreator.test.iotest.similaritymatrixwritertest import (
    SimilarityMatrixWriterTestCase,
)
from solventmapcreator.test.iotest.similaritymatrixreadertest import (
    SimilarityMatrixReaderTestCase,
)
from solventmapcreator.test.iotest.linkagewritertest import LinkageWriterTestCase
from solventmapcreator.test.iotest.solvationmapwritertest import (
    SolvationMapWriterTestCase,
)
from solventmapcreator.test.iotest.solvationenergyextractiontest import (
    SolvationEnergyExtractionTestCase,
)
from solventmapcreator.test.iotest.polynomialfilecompactortest import (
    PolynomialFileCompactorTestCase,
)
from solventmapcreator.test.solvationcalculationtest.solvationcalculatortest import (
    SolvationCalculatorTestCase,
)
from solventmapcreator.test.solvationcalculationtest.solvationmapgeneratortest import (
    SolvationMapGeneratorTestCase,
)
from solventmapcreator.test.solvationcalculationtest.solvationplotinformationtest import (
    SolvationPlotInformationTestCase,
)
from solventmapcreator.test.solvationcalculationtest.fractionaloccupancycalculatortest import (
    FractionalOccupancyCalculatorTestCase,
)
from solventmapcreator.test.solvationcalculationtest.gammacalculatortest import (
    GammaCalculatorTestCase,
)
from solventmapcreator.test.solvationcalculationtest.fgipmakertest import (
    FGIPMakerTestCase,
)
from solventmapcreator.test.solvationcalculationtest.interactionenergycalculatortest import InteractionEnergyCalculatorTestCase
from solventmapcreator.test.polynomialanalysistest.polynomialdataanalysistest import (
    PolynomialDataAnalysisTestCase,
)
from solventmapcreator.test.polynomialanalysistest.polynomialcomparisontest import (
    PolynomialComparisonTestCase,
)
from solventmapcreator.test.polynomialanalysistest.polynomialplottingtest import (
    PolynomialPlottingTestCase,
)
from solventmapcreator.test.polynomialanalysistest.polynomialvaluecalculatortest import (
    PolynomialValueCalculatorTestCase,
)
from solventmapcreator.test.polynomialanalysistest.multidimensionclusteringtest import (
    MultiDimensionClusteringTestCase,
)
from solventmapcreator.test.clusteringanalysistest.matrixinputtest import (
    MatrixInputTestCase,
)
from solventmapcreator.test.clusteringanalysistest.clustercalculationtest import (
    ClusterCalculationTestCase,
)
from solventmapcreator.test.clusteringanalysistest.clusterstatisticstest import (
    ClusterStatisticsTestCase,
)
from solventmapcreator.test.clusteringanalysistest.clusteranalysistest import (
    ClusterAnalysisTestCase,
)

logging.basicConfig()
LOGGER = logging.getLogger(__name__)
LOGGER.setLevel(logging.WARN)

IO_TEST_CASES = [
    SolvationEnergyReaderTestCase,
    SolventXMLReaderTestCase,
    PolynomialDataWriterTestCase,
    PolynomialDataReaderTestCase,
    SimilarityMatrixReaderTestCase,
    SimilarityMatrixWriterTestCase,
    LinkageWriterTestCase,
    SolvationMapWriterTestCase,
    SolvationEnergyExtractionTestCase,
    PolynomialFileCompactorTestCase,
]

SOLVATION_CALCULATION_TEST_CASES = [
    SolvationCalculatorTestCase,
    SolvationMapGeneratorTestCase,
    SolvationPlotInformationTestCase,
    FractionalOccupancyCalculatorTestCase,
    GammaCalculatorTestCase,
    FGIPMakerTestCase,
    InteractionEnergyCalculatorTestCase,
]

POLYNOMIAL_ANALYSIS_TEST_CASES = [
    PolynomialDataAnalysisTestCase,
    PolynomialComparisonTestCase,
    PolynomialPlottingTestCase,
    PolynomialValueCalculatorTestCase,
    MultiDimensionClusteringTestCase,
]

CLUSTERING_ANALYSIS_TEST_CASES = [
    MatrixInputTestCase,
    ClusterCalculationTestCase,
    ClusterStatisticsTestCase,
    ClusterAnalysisTestCase,
]


def create_test_suite():
    """Function creates a test suite and then loads all the tests from the
    different test cases.
    """
    LOGGER.info("setting up loader and test suite")
    loader = unittest.TestLoader()
    suite = unittest.TestSuite()
    for test_case in IO_TEST_CASES:
        LOGGER.debug("Adding %s", test_case)
        suite.addTests(loader.loadTestsFromTestCase(test_case))
    for test_case in SOLVATION_CALCULATION_TEST_CASES:
        LOGGER.debug("Adding %s", test_case)
        suite.addTests(loader.loadTestsFromTestCase(test_case))
    for test_case in POLYNOMIAL_ANALYSIS_TEST_CASES:
        LOGGER.debug("Adding %s", test_case)
        suite.addTests(loader.loadTestsFromTestCase(test_case))
    for test_case in CLUSTERING_ANALYSIS_TEST_CASES:
        LOGGER.debug("Adding %s", test_case)
        suite.addTests(loader.loadTestsFromTestCase(test_case))
    return suite


def run_tests():
    """Run test suite. Exits if there is a failure.

    Returns
    -------
    None
    """
    LOGGER.info("calling test suite method")
    suite = create_test_suite()
    LOGGER.info("running test suite")
    ret = (
        not unittest.TextTestRunner(verbosity=2, stream=sys.stderr)
        .run(suite)
        .wasSuccessful()
    )
    if ret:
        sys.exit(ret)


if __name__ == "__main__":
    run_tests()
